@@ -1,8 +1,8 @@
 /* gdb_interface.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002-2015,2018 David Anderson
- * Copyright (C) 2002-2015,2018 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002-2015,2018-2019 David Anderson
+ * Copyright (C) 2002-2015,2018-2019 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -831,6 +831,11 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 	if (write)
 		return FALSE;
 
+	if (!(pc->cur_req)) {
+		return(readmem(addr, KVADDR, buf, len, 
+			"gdb_readmem_callback", RETURN_ON_ERROR));
+	}
+
 	if (pc->cur_req->flags & GNU_NO_READMEM)
 		return TRUE;
 
