@@ -1,8 +1,8 @@
 /* main.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002-2017 David Anderson
- * Copyright (C) 2002-2017 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002-2019 David Anderson
+ * Copyright (C) 2002-2019 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@ static void check_xen_hyper(void);
 static void show_untrusted_files(void);
 static void get_osrelease(char *);
 static void get_log(char *);
-static char *no_vmcoreinfo(const char *);
 
 static struct option long_options[] = {
         {"memory_module", required_argument, 0, 0},
@@ -228,7 +227,8 @@ main(int argc, char **argv)
 				}
 			} else if (STREQ(long_options[option_index].name, "kaslr")) {
 				if (!machine_type("X86_64") &&
-				    !machine_type("ARM64") && !machine_type("X86"))
+				    !machine_type("ARM64") && !machine_type("X86") &&
+				    !machine_type("S390X"))
 					error(INFO, "--kaslr not valid "
 						"with this machine type.\n");
 				else if (STREQ(optarg, "auto"))
@@ -1086,6 +1086,11 @@ setup_environment(int argc, char **argv)
 	 */
 	fp = stdout;
 
+	if (!set_error("default")) {
+		fprintf(stderr, "crash: cannot malloc error() path string\n");
+		clean_exit(1);
+	}
+
 	/*
 	 *  Start populating the program_context structure.  It's used so
 	 *  frequently that "pc" has been declared globally to point to the
@@ -1101,14 +1106,20 @@ setup_environment(int argc, char **argv)
 	pc->flags2 |= REDZONE;
 	pc->confd = -2;
 	pc->machine_type = MACHINE_TYPE;
-	pc->readmem = read_dev_mem;      /* defaults until argv[] is parsed */
-	pc->writemem = write_dev_mem;
+	if (file_exists("/dev/mem", NULL)) {     /* defaults until argv[] is parsed */
+		pc->readmem = read_dev_mem;
+		pc->writemem = write_dev_mem;
+	} else if (file_exists("/proc/kcore", NULL)) {
+		pc->readmem = read_proc_kcore;
+		pc->writemem = write_proc_kcore;
+	}
 	pc->read_vmcoreinfo = no_vmcoreinfo;
 	pc->memory_module = NULL;
 	pc->memory_device = MEMORY_DRIVER_DEVICE;
 	machdep->bits = sizeof(long) * 8;
 	machdep->verify_paddr = generic_verify_paddr;
 	machdep->get_kvaddr_ranges = generic_get_kvaddr_ranges;
+	machdep->is_page_ptr = generic_is_page_ptr;
 	pc->redhat_debug_loc = DEFAULT_REDHAT_DEBUG_LOCATION;
 	pc->cmdgencur = 0;
 	pc->cmd_table = linux_command_table;
@@ -1360,6 +1371,9 @@ dump_program_context(void)
         if (pc->flags & DISKDUMP)
                 sprintf(&buf[strlen(buf)],
                         "%sDISKDUMP", others++ ? "|" : "");
+        if (pc->flags & VMWARE_VMSS)
+                sprintf(&buf[strlen(buf)],
+                        "%sVMWARE_VMSS", others++ ? "|" : "");
         if (pc->flags & SYSMAP)
                 sprintf(&buf[strlen(buf)],
                         "%sSYSMAP", others++ ? "|" : "");
@@ -1716,6 +1730,8 @@ dump_program_context(void)
 		pc->scope ? "" : "(not set)");
 	fprintf(fp, "   nr_hash_queues: %ld\n", pc->nr_hash_queues);
 	fprintf(fp, "  read_vmcoreinfo: %lx\n", (ulong)pc->read_vmcoreinfo);
+	fprintf(fp, "         error_fp: %lx\n", (ulong)pc->error_fp);
+	fprintf(fp, "       error_path: %s\n", pc->error_path);
 }
 
 char *
@@ -1941,7 +1957,7 @@ get_log(char *dumpfile)
 }
 
 
-static char *
+char *
 no_vmcoreinfo(const char *unused)
 {
 	return NULL;
